%deathstar
% Program which generates a unit sphere with a texture mapped 'Death star' surface.
% On this is overlaid a circle, drawn on the spherical surface.
% Left and right arrows control the longitude of the circle normal, and up
% and down arrow keys control the circle latitude. Keys a and z grow or
% shrink the opening angle of the circle. Minimum is fixed at zero
% (obviously!) and maximum is 180 degrees.
%
% k grows the depth of a parabolic dip in the circle.
% m shrinks it.
%
% Press p to print and q to quit.
%
% LAST UPDATED by Andy French January 2017

function deathstar

%% Set Defaults %%

%Define global variable d to share between functions
global d

%Fontsize
d.fsize = 16;

%Initial view location in [long, lat]
d.view = [60,19];

%Default half angle of circle /radians
d.alpha = pi/12;

%Angle step /radians
d.alphastep = pi/180;

%Depth of parabola (fraction of radius)
d.k = 0.2;

%Initial latitude /radians of circle normal
d.lat = 0.37;

%Initial longitude /radians of circle
d.long = -1.1;

%Latitude and longitude step sizes /radians
d.latstep = pi/30;
d.longstep = pi/30;

%k parameter step size
d.kstep = 0.05;

%Number of points for circle
d.N = 300;

%Increments for lines of longitude and latitude
d.lat_increment = (1/20)*pi;
d.long_increment = (1/20)*pi;

%Show Death Star?
d.show_DS = 1;

%

%% Initial setup %%

%Create figure
d.fig = figure('name','spherecircle','color',[1 1 1],'KeyPressFcn',@keypress,...
    'units','normalized','position',[0,0,1,1],'numbertitle','off');

%Load Earth bitmap
d.I = imread('death star texture.jpg');
s = size(d.I);
d.I = double(d.I)/255;
d.I = flipdim(d.I,1);

%Define x,y,z coordinates of unit sphere
lat = linspace(-pi/2,pi/2,s(1));
long = linspace(-pi,pi,s(2));
[long,lat] = meshgrid(long,lat);
d.x = cos(lat).*cos(long);
d.y = cos(lat).*sin(long);
d.z = sin(lat);

%Draw initial scene
d.new_plot=1;
update_plot(d.new_plot);

%%

%Creat parabolic indenation in sphere
function [x,y,z] = sphereindent(x,y,z,k,lat,long,alpha )

%Define normal in x,y,z
[xn,yn,zn] = latlong2xyz( lat, long, 1 );
xn = xn*ones(1,numel(x));
yn = yn*ones(1,numel(x));
zn = zn*ones(1,numel(x));

%Work out angular differences betwen normal and x,y,z coordinates of
%sphere. When less than alpha, reduce radius.
xx = reshape(x,[1,numel(x)]);
yy = reshape(y,[1,numel(x)]);
zz = reshape(z,[1,numel(x)]);
nu = abs( acos( dot( [xx;yy;zz],[xn;yn;zn],1 ) ));
dr = k*( 1 - ( sin(nu)/sin(alpha) ).^2 );
i = nu < alpha;
x(i) = x(i) .* (1 - dr(i) );
y(i) = y(i) .* (1 - dr(i) );
z(i) = z(i) .* (1 - dr(i) );

%%

%latlong2xyz
% Converts latitude and longitude coordinates (radians) to x,y,z
% coordinates on the surface of a sphere of radius r
function [x,y,z] = latlong2xyz( lat, long, r )
x = r.*cos(lat).*cos(long);
y = r.*cos(lat).*sin(long);
z = r.*sin(lat);

%%

%latlong2tangentxyz
% Determines x,y,z unit vectors in local tangent plane to sphere surface.
% z is a radial, y points to 'North' and x completes a right handed set.
function [xhat,yhat,zhat] = latlong2tangentxyz( lat, long )
xhat = [ -sin(long) ; cos(long) ; 0 ];
yhat = [ -cos(long)*sin(lat) ; -sin(long)*sin(lat) ; cos(lat) ];
zhat = [cos(long)*cos(lat) ; sin(long)*cos(lat) ; sin(lat) ];

%%

%spherecircle
% Defines Cartesian coordinates of a circle on the surface of a sphere.
function [x,y,z] = sphere_circle( lat, long, R, alpha, N)

%Define circle polar angle
g = linspace(0,2*pi,N);

%Define unit vectors of tangent plane
[xhat,yhat,zhat] = latlong2tangentxyz( lat, long );

%Define circle coordinates as a vector equation
xhat = repmat(xhat,[1,N]);
yhat = repmat(yhat,[1,N]);
zhat = repmat(zhat,[1,N]);
r = R*sin(alpha)*( [cos(g) ; cos(g) ; cos(g) ].*xhat +...
    [sin(g) ; sin(g) ; sin(g) ].*yhat ) + R*cos(alpha)*zhat;

%Determine outputs
x = r(1,:);
y = r(2,:);
z = r(3,:);

%%

%Function which executes when a key is pressed
function keypress(fig,evnt)
global d

if strcmp(get(fig,'currentkey'),'p')==1
    
    %Print .png file
    [lat,long] = latlongwrap(d.lat,d.long);
    filename = ['Death Star alpha=',num2str(180*d.alpha/pi,3),' k=',num2str(d.k,2),' lat=',...
        num2str(180*lat/pi,3),' long=',num2str(long*180/pi,3)];
    filename = strrep( filename,'.','p' );
    print(gcf,[filename,'.png'],'-dpng','-r300');
    
elseif strcmp(get(fig,'currentkey'),'z')==1
    
    %Reduce size of circle angle
    d.alpha = d.alpha - d.alphastep;
    if d.alpha < 0
        d.alpha = 0;
    end
    
elseif strcmp(get(fig,'currentkey'),'a')==1
    
    %Increase size of circle angle
    d.alpha = d.alpha + d.alphastep;
    if d.alpha > pi/2
        d.alpha = pi/2;
    end
    
elseif strcmp(get(fig,'currentkey'),'v')==1
    
    %Restore textured surface
    d.show_DS = 1;
    d.new_plot=1;
    
elseif strcmp(get(fig,'currentkey'),'h')==1
    
    %Hide textured surface (this speeds up redraw!). Plot a small green
    %sphere instead.
    d.show_DS = 0;
    d.new_plot = 1;
    
elseif strcmp(get(fig,'currentkey'),'m')==1
    
    %Reduce size of indentation k parameter
    d.k = d.k - d.kstep;
    
elseif strcmp(get(fig,'currentkey'),'k')==1
    
    %Increase size of indentation k parameter
    d.k = d.k + d.kstep;
    
elseif strcmp(get(fig,'currentkey'),'uparrow')==1
    
    %Increase circle normal latitude
    d.lat = d.lat + d.latstep;
    
elseif strcmp(get(fig,'currentkey'),'downarrow')==1
    
    %Decrease circle normal latitude
    d.lat = d.lat - d.latstep;
    
elseif strcmp(get(fig,'currentkey'),'leftarrow')==1
    
    %Increase circle normal longitude
    d.long = d.long - d.longstep;
    
elseif strcmp(get(fig,'currentkey'),'rightarrow')==1
    
    %Increase circle normal longitude
    d.long = d.long + d.longstep;
    
elseif strcmp(get(fig,'currentkey'),'q')==1
    
    %Close figure
    close(d.fig);
    return
end

%Update circle calculations and update plot
update_plot(d.new_plot);
d.new_plot = 0;

%%

%Update plot
function update_plot(np)
global d
if np==1
    cla;
end

%Plot sphere and texture-map Death Star onto it
if d.show_DS == 1
    
    %Plot sphere surface
    if np==1
        d.DS = surf(d.x,d.y,d.z,d.I);
    end
    
    %Indent sphere
    [x,y,z] = sphereindent(d.x,d.y,d.z,d.k,d.lat,d.long,d.alpha );
    set( d.DS,'xdata',x,'ydata',y,'zdata',z);
    
else
    %Plot a small sphere
    lat = linspace(-pi/2,pi/2,10);
    long = linspace(-pi,pi,20);
    [long,lat] = meshgrid(long,lat);
    x = 0.1*cos(lat).*cos(long);
    y = 0.1*cos(lat).*sin(long);
    z = 0.1*sin(lat);
    if np==1
        d.DS = surf(x,y,z);
    end
end
axis vis3d;
shading interp;
hold on;
grid on;
xlabel('x','fontsize',d.fsize);
ylabel('y','fontsize',d.fsize);
zlabel('z','fontsize',d.fsize);
set(gca,'fontsize',d.fsize);
view(d.view)
axis manual

%Lines of longitude
long = 0 : d.long_increment : 2*pi - 2*pi/10;
for n=1:length(long)
    [x,y,z] = sphere_circle( 0, long(n), 1, pi/2, 300);
    [x,y,z] = sphereindent(x,y,z,d.k,d.lat,d.long,d.alpha );
    if np==1
        d.longitudes(n) = plot3(x,y,z,'k-');
    else
        set( d.longitudes(n),'xdata',x,'ydata',y,'zdata',z);
    end
end

%

%Lines of latitude
lat = 0 : d.lat_increment : pi/2;
for n=1:length(lat)
    [x,y,z] = sphere_circle( pi/2, 0, 1, lat(n), 300);
    [x,y,z] = sphereindent(x,y,z,d.k,d.lat,d.long,d.alpha );
    if np==1
        d.latitudes1(n) = plot3(x,y,z,'k-');
    else
        set( d.latitudes1(n),'xdata',x,'ydata',y,'zdata',z);
    end
    [x,y,z] = sphere_circle( -pi/2, 0, 1, lat(n), 300);
    [x,y,z] = sphereindent(x,y,z,d.k,d.lat,d.long,d.alpha );
    if np==1
        d.latitudes2(n) = plot3(x,y,z,'k-');
    else
        set( d.latitudes2(n),'xdata',x,'ydata',y,'zdata',z);
    end
end

%

%Define spherical circle
[x,y,z] = sphere_circle( d.lat, d.long, 1, d.alpha, d.N);
if np==1
    d.circle = plot3(x,y,z,'k-','linewidth',2);
else
    set( d.circle,'xdata',x,'ydata',y,'zdata',z);
end

%

%Plot normal to circle
[x,y,z] = latlong2xyz( d.lat, d.long, 1.5 );
if np==1
    d.normal = plot3( [0,x],[0,y],[0,z],'g-','linewidth',2 );
else
    set( d.normal,'xdata',[0,x],'ydata',[0,y],'zdata',[0,z]);
end

%Define title
d.title = ['Death Star: \alpha = ',num2str(180*d.alpha/pi,3),'^o, lat = ',...
    num2str(180*d.lat/pi,3),'^o, long = ',num2str(d.long*180/pi,3),'^o, k = ',num2str(d.k,2)];
title( d.title,'fontsize',d.fsize);

%%

%Wrap lat, long into correct range
function [lat,long] = latlongwrap(lat,long)

%Remove multiples of 2*pi
lat = rem( lat, 2*pi);
long = rem( long, 2*pi);

%Give correct latitudes
if lat > 0
    if ( lat <= pi ) && ( lat > pi/2 )
        lat = pi - lat;
        long = long + pi;
    elseif ( lat > pi ) && ( lat < 3*pi/2 )
        lat = - (lat - pi );
        long = long + pi;
    elseif ( lat >= 3*pi/2 ) && ( lat < 2*pi )
        lat = - (2*pi - lat );
    end
else
    if ( lat <= -pi/2 ) && ( lat > -pi )
        lat = - (pi + lat );
        long = long + pi;
    elseif ( lat <= -pi ) && ( lat > -3*pi/2 )
        lat = - (pi + lat) ;
        long = long + pi;
    elseif    ( lat > -2*pi ) && ( lat <= -3*pi/2 )
        lat = 2*pi + lat;
    end
end

%Give correct longitudes
if long > pi
    long = -(2*pi - long );
elseif long < -pi
    long = 2*pi + long;
end

%End of code