%spolartraj
% Program which generates a unit sphere with a texture mapped Earth
% surface. On this is overlaid a trajectory defined parametrically in
% spherical polar coordinates (i.e. in terms of range, latitude and
% longitude).
%
% Press p to print and q to quit.
%
% LAST UPDATED by Andy French January 2017

function spolartraj

%% Set Defaults %%

%Define global variable d to share between functions
global d

%Parametric trajectory (in terms of variable t)
d.tmin = 0;
d.tmax = 4*pi;
d.rt = '1.3 + 0.1*sin(t)';
d.latt = 't + 0.1*cos(3*t)';
d.longt = '2*t';

%Fontsize
d.fsize = 14;

%Initial view location in [long, lat]
d.view = [140,35];

%Number of points for parametric trajectory
d.N = 1000;

%Increments for lines of longitude and latitude
d.lat_increment = (1/20)*pi;
d.long_increment = (1/20)*pi;

%

%Create figure
d.fig = figure('name','spherecircle','color',[1 1 1],'KeyPressFcn',@keypress,...
    'units','normalized','position',[0,0,1,1],'numbertitle','off');

%Load Earth bitmap
I = imread('death star texture.jpg');
I = imread('earth_reallylowres.jpg');
s = size(I);
I = double(I)/255;
I = flipdim(I,1);

%Define x,y,z coordinates of unit sphere
lat = linspace(-pi/2,pi/2,s(1));
long = linspace(-pi,pi,s(2));
[long,lat] = meshgrid(long,lat);
x = cos(lat).*cos(long);
y = cos(lat).*sin(long);
z = sin(lat);

%Plot sphere and texture-map Earth onto it
surf(x,y,z,I);
axis vis3d;
shading interp;
hold on;
grid on;
xlabel('x','fontsize',d.fsize);
ylabel('y','fontsize',d.fsize);
zlabel('z','fontsize',d.fsize);
set(gca,'fontsize',d.fsize);
view(d.view)
axis manual

%Lines of longitude
long = 0 : d.long_increment : 2*pi - 2*pi/10;
for n=1:length(long)
    [x,y,z] = sphere_circle( 0, long(n), 1, pi/2, 300);
    plot3(x,y,z,'w-');
end

%

%Lines of latitude
lat = 0 : d.lat_increment : pi/2;
for n=1:length(lat)
    [x,y,z] = sphere_circle( pi/2, 0, 1, lat(n), 300);
    plot3(x,y,z,'w-');
    [x,y,z] = sphere_circle( -pi/2, 0, 1, lat(n), 300);
    plot3(x,y,z,'w-');
end

%

%Define title
d.title = {
    ['r(t) = ',d.rt],...
    ['lat(t) = ',d.latt],...
    ['long(t) = ',d.longt]
    };
title( d.title,'fontsize',d.fsize,'position',[8.54,10.28,11.0]);

%

%Define trajectory in spherical polars and plot
t = linspace(d.tmin,d.tmax,d.N);
eval(['r = ',d.rt,';']);
eval(['lat = ',d.latt,';']);
eval(['long = ',d.longt,';']);
x = r.*cos(long).*cos(lat);
y = r.*sin(long).*cos(lat);
z = r.*sin(lat);
plot3(x,y,z,'r','linewidth',2);
plot3(x(1),y(1),z(1),'g*','markersize',2);
plot3(x(end),y(end),z(end),'r*','markersize',2);

%%

%Function which executes when a key is pressed
function keypress(fig,evnt)
global d

if strcmp(get(fig,'currentkey'),'p')==1
    
    %Print .png file
    filename = ['r=',d.rt,...
    ' lat=',d.latt,...
    ' long=',d.longt];
    filename = strrep( filename,'.','p' );
    filename = strrep( filename,'*','' );
    print(gcf,[filename,'.png'],'-dpng','-r300');
    
elseif strcmp(get(fig,'currentkey'),'q')==1
    
    %Close figure
    close(d.fig);
    return
end

%%

%latlong2tangentxyz
% Determines x,y,z unit vectors in local tangent plane to sphere surface.
% z is a radial, y points to 'North' and x completes a right handed set.
function [xhat,yhat,zhat] = latlong2tangentxyz( lat, long )
xhat = [ -sin(long) ; cos(long) ; 0 ];
yhat = [ -cos(long)*sin(lat) ; -sin(long)*sin(lat) ; cos(lat) ];
zhat = [cos(long)*cos(lat) ; sin(long)*cos(lat) ; sin(lat) ];

%%

%spherecircle
% Defines Cartesian coordinates of a circle on the surface of a sphere.
function [x,y,z] = sphere_circle( lat, long, R, alpha, N)

%Define circle polar angle
g = linspace(0,2*pi,N);

%Define unit vectors of tangent plane
[xhat,yhat,zhat] = latlong2tangentxyz( lat, long );

%Define circle coordinates as a vector equation
xhat = repmat(xhat,[1,N]);
yhat = repmat(yhat,[1,N]);
zhat = repmat(zhat,[1,N]);
r = R*sin(alpha)*( [cos(g) ; cos(g) ; cos(g) ].*xhat +...
    [sin(g) ; sin(g) ; sin(g) ].*yhat ) + R*cos(alpha)*zhat;

%Determine outputs
x = r(1,:);
y = r(2,:);
z = r(3,:);

%%
    
%End of code